/*
swapper  copyright 2003 Jan Panteltje.
swaps randomly characters in a word, leaving the first and last in place.


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "cswap.h"

char * strfry (char *STRING);
/*
strfry
======

   The function below addresses the perennial programming quandary:
"How do I take good data in string form and painlessly turn it into
garbage?"  This is actually a fairly simple task for C programmers who 
do not use the GNU C library string functions, but for programs based
on the GNU C library, the trfry' function is the preferred method for
destroying string data.

   The prototype for this function is in tring.h'.

 - Function: char * strfry (char *STRING)
     trfry' creates a pseudorandom anagram of a string, replacing the
     input with the anagram in place.  For each position in the string,
     trfry' swaps it with a position in the string selected at random
     (from a uniform distribution).  The two positions may be the same.

     The return value of trfry' is always STRING.

     *Portability Note:*  This function is unique to the GNU C library.
*/


int print_usage()
{
fprintf(stdout, "\
Usage:             cat infile.txt | cswap > outfile.txt\n\n\
-d                 debug (prints functions and arguments)\n\
-h                 help (this help)\n\
");

return 1;
} /* end function print_usage */



int main(int argc, char **argv)
{
int a, c, d, i;
FILE *ifptr;
FILE *ofptr;
char word[4096];
int char_count;

//fprintf(stdout, "\nPanteltje cswap-%s\n\n", VERSION);

/* defaults */
/* end defaults */

/* seed the random number generator with somthing that changes constantly... */
srand( time(0) );

/* process command line options */
while(1)
    {
    a = getopt(argc, argv, "dhi:");
    if(a == -1) break;

    switch(a)
        {
        case 'd':
            debug_flag = 1;
            break;
        case 'h':
            print_usage();
            exit(1);
            break;
        case -1:
            break;
        case '?':
            if (isprint(optopt) )
                 {
                 fprintf(stdout, "Unknown option `-%c'.\n", optopt);
                 }
            else
                {
                fprintf(stdout, "Unknown option character `\\x%x'.\n",\
                optopt);
                }
            print_usage();

            exit(1);
            break;            
        default:
            print_usage();

            exit(1);
            break;
        }/* end switch a */
    }/* end while getopt() */

/* report special situations */

/* open input */
ifptr = fopen("/dev/stdin", "r");
if(! ifptr)
    {
    fprintf(stderr, "cswap: could not open /dev/stdin for read, aborting.\n");

    exit(1);
    }

/* open output */
ofptr = fopen("/dev/stdout", "w");
if(! ofptr)
    {
    fprintf(stderr, "cswap: could not open /dev/stdout for write, aborting.\n");

    exit(1);
    }

/* read from input, write to output */
char_count = 0;
while(1)
    {
    /* wait for start word, copy white space to output if not yet found */
    while(1)
        {
        c = getc(ifptr);
        if(c == EOF)
            {
            fclose(ifptr);
            fclose(ofptr);

            /* return OK */
            exit(0);    
            }

		if( isalnum(c) ) break;

//        if(! isspace(c) ) break;
//		if(c == 10) break; // LF, next character on line is start of word

//		/* this is better if scrambling code like debug_flag, else the last part stays as it is */
//		if(c != '_') break;

        a = fwrite(&c, sizeof(char), 1, ofptr);
        if(a != 1)
            {
            fprintf(stderr, "cswap: could not write to output, aborting\n");

            exit(1);
            }
        }

    word[0] = c;
    char_count = 1;

    /* read word in array */
    for(i = 1; i < 4096; i++)
        {
        c = getc(ifptr);
        if(c == EOF)
            {
            fclose(ifptr);
            fclose(ofptr);

            /* return OK */
            exit(0);    
            }

//        if(! isalpha(c) ) break;
        if(! isalnum(c) ) break;     

        word[char_count] = c;
        char_count++;
        }

    if(c == 4096)
        {
        fprintf(stderr, "cswap(): word too long!, aborting\n");

        exit(1);
        }

    /* shuffle word from second character to one before last character */
    /* save last character */
    d = word[char_count - 1];

    /* put a string termination '0' in place of last character */
    word[char_count - 1] = 0;

    /* fry the string (see libc.info) */
//    char * strfry (char *STRING)
    strfry(word + 1);

    /* replace the last 0 we put there with the original character */
    word[char_count - 1] = d;

    /* output word */
    a = fwrite(word, sizeof(char), char_count, ofptr);  
    if(a != char_count)
        {
        fprintf(stderr, "cswap: could not write to output, aborting\n");

        exit(1);
        }

    /* output space */
    a = fwrite(&c,  sizeof(char), 1, ofptr);
    if(a != 1)
        {
        fprintf(stderr, "cswap: could not write to output, aborting\n");

        exit(1);
        }

    }

/* return OK */
exit(0);
} /* end function main */


